/*
 * callbacks.c
 * Copyright (C) 2003 footprints <footprints at lycos dot it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Add the timer */
gboolean timerEvent();
/* Get the ip address */
gboolean get_ip();

/* monitor widget */
static GtkWidget* gpppoem = NULL;

static gint root_x=0;
static gint root_y=0;

static gchar interface[6]="ppp0";
static int rx_b, tx_b;
static gboolean reset=TRUE;

#include "eggtrayicon.h"

static void docklet_x11_create(GtkWidget* widget);
static EggTrayIcon *docklet = NULL;
static GtkWidget *image = NULL;

/*
 *	Add the monitor, the timer and the docklet
 */
void add_my_interface(GtkWidget* menu_widget)
{
	/* create the monitor */
	gpppoem=create_gpppoem();

	/* add the timeout */
	g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 1000, timerEvent, 0, 0);
	get_ip();

	/* create the docklet */
	GtkIconFactory *icon_factory;
	GtkWidget *win;
	GdkPixbuf* pixbuf;
	GtkIconSet* iconset;
	gchar* pathname;

	docklet_x11_create(menu_widget);

	icon_factory = gtk_icon_factory_new();
	gtk_icon_factory_add_default(icon_factory);

	pathname = gnome_program_locate_file(NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
					"gpppoem/gppp.png", TRUE, NULL);
	if (!pathname){
		g_warning (_("Couldn't find pixmap file: %s"), "gpppoem/gppp.png");
	}
	pixbuf = gdk_pixbuf_new_from_file(pathname, NULL);
	iconset = gtk_icon_set_new_from_pixbuf(pixbuf);

	gtk_icon_factory_add(icon_factory, "gppp", iconset);
	gtk_icon_set_unref(iconset);

	/* register logo icon size */
	gtk_icon_size_register("gppp", 210, 150);
	g_object_unref(G_OBJECT(icon_factory));

	gtk_image_set_from_stock(GTK_IMAGE(image), "gppp", GTK_ICON_SIZE_LARGE_TOOLBAR);

}

/*
 *	Add the timer
 */
gboolean timerEvent()
{
	gchar* buf=g_malloc(1);
	gsize size;
	gchar* str_ppp=0;
	int i;
	gchar** str_ppp_split=0;
	gdouble rx=0;
	gdouble tx=0;
	gchar* received=0;
	gchar* transmitted=0;
	GtkWidget* label=0;

	g_file_get_contents("/proc/net/dev", &buf, &size, 0);

	if(!(str_ppp=g_strrstr(buf, interface))){
		GtkWidget* label=0;
		if(label=lookup_widget(gpppoem, "label_rx_b"))
			gtk_label_set_text(GTK_LABEL(label), "0");
		label=0;
		if(label=lookup_widget(gpppoem, "label_tx_b"))
			gtk_label_set_text(GTK_LABEL(label), "0");
		return TRUE;
	}
	/* remove white spaces */
	for(i=0; i<strlen(str_ppp)-1; i++){
		if(str_ppp[i]=='\t'){
			str_ppp[i]=' ';
			i--;
		}
		else if((str_ppp[i]==' ') && (str_ppp[i+1]==' ')){
			strcpy(str_ppp+i, str_ppp+i+1);
			i--;
		}
	}
	/* split the string */
	str_ppp_split=g_strsplit(str_ppp+5, " ", 16);
	if(*(str_ppp+5)==' '){
		rx=g_strtod(str_ppp_split[1], 0);
		tx=g_strtod(str_ppp_split[9], 0);
	}
	else{
		rx=g_strtod(str_ppp_split[0], 0);
		tx=g_strtod(str_ppp_split[8], 0);
	}

	if(reset==TRUE){
		rx_b=rx;
		tx_b=tx;
		reset=FALSE;
	}
	received=g_malloc(100);
	if(rx > 1048576.0)
		sprintf(received, "%.0f (%.2f Mb) at %.1f Kb/sec", rx, rx/(1048576.0), (rx-rx_b)/(1024.0));
	else if(rx > 1024.0)
		sprintf(received, "%.0f (%.2f Kb) at %.1f Kb/sec", rx, rx/(1024.0), (rx-rx_b)/(1024.0));
	else sprintf(received, "%.0f bytes at %.1f Kb/sec", rx, (rx-rx_b)/(1024.0));

	transmitted=g_malloc(100);
	if(tx > 1048576.0)
		sprintf(transmitted, "%.0f (%.2f Mb) at %.1f Kb/sec", tx, tx/(1048576.0), (tx-tx_b)/(1024.0));
	else if(tx > 1024.0)
		sprintf(transmitted, "%.0f (%.2f Kb) at %.1f Kb/sec", tx, tx/(1024.0), (tx-tx_b)/(1024.0));
	else sprintf(transmitted, "%.0f bytes at %.1f Kb/sec", tx, (tx-tx_b)/(1024.0));

	rx_b=rx;
	tx_b=tx;

	if(label=lookup_widget(gpppoem, "label_rx_b"))
		gtk_label_set_text(GTK_LABEL(label), received);
	label=0;
	if(label=lookup_widget(gpppoem, "label_tx_b"))
		gtk_label_set_text(GTK_LABEL(label), transmitted);

	g_strfreev(str_ppp_split);
	g_free(buf);
	return TRUE;
}
/*
 *	Get the ip address
 */
gboolean get_ip()
{
	GtkWidget* label=0;
	FILE* pout=0;

	label=lookup_widget(gpppoem, "label_ip");

	gchar* command_line=g_malloc(81);

	g_snprintf(command_line, 80, "gpppoem.sh interface %s ", interface);
	if(system(command_line)){
		gtk_label_set_text(GTK_LABEL(label), "Device not found");
		g_free(command_line);
		return FALSE;
	}

	g_snprintf(command_line, 80, "gpppoem.sh status %s ", interface);
	pout=popen(command_line, "r");
	g_free(command_line);

	gchar* str_ip=g_malloc(16);

	fscanf(pout, "%s", str_ip);
	pclose(pout);

	gtk_label_set_text(GTK_LABEL(label), str_ip);

	g_free(str_ip);

	return TRUE;
}

void
on_item_ppp0_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	strcpy(interface, "ppp0");
	reset=TRUE;
	get_ip(menuitem);
}


void
on_item_eth0_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	strcpy(interface, "eth0");
	reset=TRUE;
	get_ip(menuitem);
}


void
on_item_ppp1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	strcpy(interface, "ppp1");
	reset=TRUE;
	get_ip(menuitem);
}


void
on_item_eth1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	strcpy(interface, "eth1");
	reset=TRUE;
	get_ip(menuitem);
}

void
on_item_lo_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	strcpy(interface, "lo");
	reset=TRUE;
	get_ip(menuitem);
}

void
on_gpppoem_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_window_get_position(GTK_WINDOW(gpppoem), &root_x, &root_y);
	gtk_widget_hide (gpppoem);
}


void
on_item_about_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* about = create_about ();
	gtk_widget_show (about);
}


void
on_item_close_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}

void
on_item_connect_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(popen("gpppoem.sh connect", "r")<0){
		g_warning("Error connecting!\n");
	}
}

void
on_item_disconnect_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(popen("gpppoem.sh dconnect", "r")<0){
		g_warning("Error disconnecting!\n");
	}
}

void
on_item_monitor_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(GTK_WIDGET_VISIBLE(gpppoem)){
		gtk_window_get_position(GTK_WINDOW(gpppoem), &root_x, &root_y);
		gtk_widget_hide (gpppoem);
	}
	else{
		gtk_window_move(GTK_WINDOW(gpppoem), root_x, root_y);
		gtk_widget_show (gpppoem);
	}
}



/*
 *	Docklet code
 */
static void
docklet_x11_clicked_cb(GtkWidget *button, GdkEventButton *event, void *data)
{
	GtkWidget* menu_widget;

	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 1)){
		if(GTK_WIDGET_VISIBLE(gpppoem)){
			gtk_window_get_position(GTK_WINDOW(gpppoem), &root_x, &root_y);
			gtk_widget_hide (gpppoem);
		}
		else{
			gtk_window_move(GTK_WINDOW(gpppoem), root_x, root_y);
			gtk_widget_show (gpppoem);
		}
	}
	else if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)){
		if(menu_widget=lookup_widget(data, "menu")){
			gtk_menu_popup (GTK_MENU(menu_widget), NULL, NULL, NULL, NULL,
					event->button, event->time);
		}
	}
}

/* Docklet code based on docklet plugin for Gaim, copyright Robert McQueen */
static void
docklet_x11_destroyed_cb(GtkWidget *widget, void *data)
{
	g_object_unref(G_OBJECT(docklet));
	docklet = NULL;
}

static void
docklet_x11_destroy()
{
	g_signal_handlers_disconnect_by_func(G_OBJECT(docklet), G_CALLBACK(docklet_x11_destroyed_cb), NULL);
	gtk_widget_destroy(GTK_WIDGET(docklet));

	g_object_unref(G_OBJECT(docklet));
	docklet = NULL;
}

static void
docklet_x11_create(GtkWidget* menu_widget)
{
	GtkWidget *box;

	docklet = egg_tray_icon_new("gpppoem");
	box = gtk_event_box_new();
	image = gtk_image_new();

	g_signal_connect(G_OBJECT(docklet), "destroy", G_CALLBACK(docklet_x11_destroyed_cb), NULL);
	g_signal_connect(G_OBJECT(box), "button-press-event", G_CALLBACK(docklet_x11_clicked_cb), menu_widget);

	gtk_container_add(GTK_CONTAINER(box), image);
	gtk_container_add(GTK_CONTAINER(docklet), box);
	gtk_widget_show_all(GTK_WIDGET(docklet));

	/* ref the docklet before we bandy it about the place */
	g_object_ref(G_OBJECT(docklet));
}




